@@ -1,5 +1,5 @@
 require 'socket'
-require 'kconv'
+require 'nkf'
 
 module DIP
   module IPMessenger
@@ -212,6 +212,14 @@ module DIP
       def each
 	@ary.reverse_each {|x| yield(x)}
       end
+
+      def has_unopened?
+	@ary.each do |x|
+	  next if x.sent?
+	  return true unless x.opened
+	end
+	return false
+      end
     end
 
     class Seq
@@ -254,10 +262,11 @@ module DIP
       attr_accessor :members, :broadcast_addr
       attr_reader :port
       attr_reader :addr
+      attr_reader :absence_msg
 
       def kconv(str)
 	return nil if str == nil
-	Kconv::toeuc(str.to_s)
+	NKF.nkf('-edXm0', str.to_s)
       end
 
       def log(msg, flag=nil)
@@ -307,7 +316,8 @@ module DIP
 
       def telegram(command, supplement)
 	seq = @packet_no.succ
-	str = Kconv::tosjis("#{PROTCOL_VER}:#{seq}:#{@username}:#{@hostname}:#{command}:#{supplement}")
+	str = "#{PROTCOL_VER}:#{seq}:#{@username}:#{@hostname}:#{command}:#{supplement}"
+	str = NKF.nkf('-sdXm0', str)
 	return seq, str
       end
 
@@ -315,14 +325,22 @@ module DIP
 	mode = secret ? IPMSG_SECRETOPT : 0
 	seq, buf = telegram(IPMSG_SENDMSG|mode, msg)
 	@msg_holder.add_sent(host, @port, msg, seq)
-	@socket.send(buf, 0, host, @port)
+	begin
+	  @socket.send(buf, 0, host, @port)
+	rescue SystemCallError
+	  log([$!, $@].inspect)
+	end
 	seq
       end
 
       def send(addr, port, command, supplement)
 	seq, msg = telegram(command, supplement)
 	log(">>>>>Send to #{addr} - \"#{msg}\"\n")
-	@socket.send(msg, 0, addr, port)
+	begin
+	  @socket.send(msg, 0, addr, port)
+	rescue SystemCallError
+	  log([$!, $@].inspect)
+	end
 	seq
       end
 
@@ -331,7 +349,11 @@ module DIP
 	log(">>>>>Broadcast message - \"#{msg}\"\n")
 
 	@broadcast_addr.each do |addr|
-	  @socket.send(msg, 0, addr, @port)
+	  begin
+	    @socket.send(msg, 0, addr, @port)
+	  rescue SystemCallError
+	    log([$!, $@].inspect)
+	  end
 	end
 	seq
       end
